# -*- coding: utf-8 -*-
"""Module with client calling the OpenAI GPT completion endpoint"""
import json

import openai
import requests
from openai.error import RateLimitError, APIError, TryAgain, Timeout, APIConnectionError, ServiceUnavailableError, \
    InvalidRequestError
from retry.api import retry_call

# ==============================================================================
# CONSTANT DEFINITION
# ==============================================================================

API_RETRY_EXCEPTIONS = (
    requests.HTTPError, RateLimitError, APIError, TryAgain, Timeout, APIConnectionError, ServiceUnavailableError)
API_CATCH_EXCEPTIONS = (
    requests.HTTPError, RateLimitError, APIError, TryAgain, Timeout, APIConnectionError, ServiceUnavailableError,
    InvalidRequestError)


# ==============================================================================
# CLASS AND FUNCTION DEFINITION
# ==============================================================================

class GPTResponse:
    def __init__(self, raw_response, response_text):
        self.raw_response = raw_response
        self.response_text = response_text


class GPTClient:
    def __init__(self, api_configuration_preset: dict, temperature: float, max_tokens: int):
        self.max_attempts = api_configuration_preset.get("max_attempts", 3)
        self.wait_interval = api_configuration_preset.get("wait_interval", 5)
        self.request_timeout = 60
        self.api_type = api_configuration_preset.get("api_type", "openai")
        self.client_params = {
            "temperature": temperature,
            "max_tokens": max_tokens
        }
        if "api_key" not in api_configuration_preset:
            raise Exception("API key must be set in plugin's settings")

        self.client_secret_params = {
            "api_key": api_configuration_preset["api_key"]
        }

        if self.api_type == 'openai':
            model = api_configuration_preset.get("engine", "gpt-3.5-turbo")
            self.client_params.update({
                "model": model,
                "api_type": "openai",
            })
            self.model_type = "chat" if model.find("turbo") >= 0 or model.find("gpt-4") >= 0 else "completion"
        elif self.api_type == 'azure':
            if "deployment_id" not in api_configuration_preset:
                raise Exception("Deployment name must be set in plugin's settings")
            if "api_base" not in api_configuration_preset:
                raise Exception("Azure OpenAI Endpoint URL must be set in plugin's settings")
            self.client_params.update({
                "deployment_id": api_configuration_preset["deployment_id"],
                "api_base": api_configuration_preset["api_base"],
                "api_type": "azure",
                "api_version": "2023-05-15"
            })
            self.model_type = api_configuration_preset.get("model_type", "chat")
        else:
            raise Exception("Unsupported API type")

    def __str__(self):
        # This SHOULD never contain the API key!
        return json.dumps({
            **self.client_params,
            "model_type": self.model_type,
            "max_attempts": self.max_attempts,
            "wait_interval": self.wait_interval,
            "request_timeout": self.request_timeout,
        }, indent=2)

    def send_prompt(self, prompt: str):
        def send_prompt_without_retry() -> GPTResponse:
            if self.model_type == "chat":
                response = openai.ChatCompletion.create(
                    **self.client_params,
                    **self.client_secret_params,
                    messages=[
                        {"role": "user", "content": prompt}
                    ]
                )
                if "choices" in response:
                    obj = response["choices"][0]
                    return GPTResponse(obj, obj.get("message", {}).get("content", ""))
                else:
                    # OpenAIs Python client seems to handle all exceptions so this should rarely be called
                    user_message = f"Encountered the following error while sending an API request to OpenAI: {response}"
                    raise requests.HTTPError(user_message)
            else:
                response = openai.Completion.create(
                    **self.client_params,
                    **self.client_secret_params,
                    prompt=prompt,
                    stop="\n"
                )

                if "choices" in response:
                    obj = response["choices"][0]
                    return GPTResponse(obj, obj.get("text", ""))
                else:
                    # OpenAIs Python client seems to handle all exceptions so this should rarely be called
                    user_message = f"Encountered the following error while sending an API request to OpenAI: {response}"
                    raise requests.HTTPError(user_message)

        return retry_call(send_prompt_without_retry, exceptions=API_RETRY_EXCEPTIONS, delay=self.wait_interval,
                          tries=self.max_attempts)
