import json
import logging
from typing import List, Tuple, Dict, Any

from gpt_api_client import GPTClient


class GPTMultiQuestionAnswererResult:
    def __init__(self, response_object, parsed_answers: Dict[str, str]):
        self.parsed_answers = parsed_answers
        self.response_object = response_object


class GPTMultiQuestionAnswerer:
    def __init__(self, gpt_client: GPTClient, questions: List[Tuple[str, str]]):
        self.gpt_client = gpt_client
        self.questions = questions

        logging.info("GPT API client params:")
        logging.info(gpt_client)

        logging.info("Prompt used for question answering:")
        logging.info(self._format_prompt("[input text]"))

    def _format_prompt(self, input_text: str) -> str:
        prompt = "You are a helpful assistant that answers questions based on a document."
        prompt += "\n"
        prompt += "\n"
        prompt += "Questions are defined in this JSON object:"
        prompt += "\n"
        prompt += json.dumps({('Q%s' % (index + 1)): question for index, (_, question) in enumerate(self.questions)},
                             indent=2)
        prompt += "\nWrite ONLY the answers in a JSON object having the same keys but with answers as values. "
        prompt += "Do NOT repeat the questions. "
        prompt += "Do not try to explain if you cannot answer."
        prompt += "\n"
        prompt += "\n"
        prompt += "Write the JSON answer object based on the document below:"
        prompt += "\n"
        prompt += json.dumps({"document": input_text}, indent=2)
        prompt += "\n"
        return prompt

    def _parse_raw_response(self, response_text):
        parsed_answers = {}
        try:
            resp_obj = json.loads(response_text)
            resp_obj = resp_obj if isinstance(resp_obj, dict) else {}
            for index, (column_name, _) in enumerate(self.questions):
                answer = resp_obj.get('Q%s' % (index + 1))
                answer = answer if isinstance(answer, str) else "unknown"
                parsed_answers[column_name] = answer
        except:
            # Not much we can do
            pass
        return parsed_answers

    def get_response(self, input_text: Any) -> GPTMultiQuestionAnswererResult:
        prompt = self._format_prompt(str(input_text))
        gpt_response = self.gpt_client.send_prompt(prompt)
        parsed_answers = self._parse_raw_response(gpt_response.response_text)
        return GPTMultiQuestionAnswererResult(gpt_response.raw_response, parsed_answers)
