import json
import logging
from typing import List, Union, Any, Dict

from gpt_api_client import GPTClient


class GPTZeroShotTextClassifierResult:
    def __init__(self, response_object, parsed_answer: Dict[str, str]):
        self.response_object = response_object
        self.predicted_class = parsed_answer["class"] if "class" in parsed_answer else None
        self.explanation = parsed_answer["explanation"] if "explanation" in parsed_answer else None


class GPTZeroShotTextClassifier:
    def __init__(self, gpt_client: GPTClient, input_desc: str, output_desc: str, classes: List[Dict[str, str]], examples: List[Dict[str, Dict[str, Union[int, str]]]], explanation_enabled: bool):
        self.gpt_client = gpt_client
        self.input_desc = input_desc
        self.output_desc = output_desc
        self.classes = classes
        self.examples = examples
        self.explanation_enabled = explanation_enabled

        logging.info("GPT API client params:")
        logging.info(gpt_client)

        logging.info("Prompt used for classification:")
        logging.info(self._format_prompt("[input text]"))

    def _format_prompt(self, input_text: str) -> str:
        prompt = "You are a helpful assistant that classifies text "
        if self.input_desc:
            prompt += f"({self.input_desc}) "

        ### Classes ###
        prompt += "into one of these classes"
        if self.output_desc:
            prompt += f" ({self.output_desc})"
        prompt += ":\n"
        prompt += json.dumps(self.classes, indent=2)
        prompt += "\n\n"

        ### Output format instructions ###
        # Note: asking to predict class name first consistently improves quality of predictions
        prompt += "No other class is allowed. "
        if self.explanation_enabled:
            prompt += "You must classify the document into exactly one of the listed classes above and give a very short explanation. Answer using only this JSON format:\n"
            prompt += "{\"explanation\": \"[short explanation]\", \"class_name\": \"[predicted class_name]\", \"class_id\": \"[predicted class_id]\"}\n\n"
        else:
            prompt += "You must classify the document into exactly one of the listed classes above. Answer using only this JSON format:\n"
            prompt += "{\"class_name\": \"[predicted class_name]\", \"class_id\": \"[predicted class_id]\"}\n\n"

        if self.examples:
            prompt += "Here are a few examples:\n"
            prompt += json.dumps(self.examples, indent=2)
            prompt += "\n\n"

        ### output start ###
        prompt += "Here is the document to classify in a JSON string:\n"
        prompt += json.dumps(input_text)
        return prompt

    def _parse_raw_response(self, response_text) -> Dict[str, str]:
        parsed_answer = {}
        try:
            resp_obj = json.loads(response_text)
            resp_obj = resp_obj if isinstance(resp_obj, dict) else {}
            if "class_id" not in resp_obj:
                return {"class": "unknown"}
            for pred_class in self.classes:
                if str(pred_class["class_id"]) == str(resp_obj["class_id"]):
                    parsed_answer["class"] = pred_class["class_name"]
                    break
            if self.explanation_enabled:
                if "explanation" in resp_obj:
                    parsed_answer["explanation"] = str(resp_obj["explanation"])
            return parsed_answer
        except:
            # Not much we can do
            pass
        return {"class": "unknown"}

    def get_response(self, input_text: Any) -> GPTZeroShotTextClassifierResult:
        prompt = self._format_prompt(str(input_text))
        gpt_response = self.gpt_client.send_prompt(prompt)
        parsed_answers = self._parse_raw_response(gpt_response.response_text)
        return GPTZeroShotTextClassifierResult(gpt_response.raw_response, parsed_answers)
