# -*- coding: utf-8 -*-
# This is a test file intended to be used with pytest
# pytest automatically runs all the function starting with "test_"
# see https://docs.pytest.org for more information
import os
from gpt_api_client import GPTClient
from question_answering import GPTMultiQuestionAnswerer
from summarization import GPTTextSummarizer
from text_classification import GPTZeroShotTextClassifier
from text_generation import GPTTextGenerator


class TestRecipe:
    client: GPTClient = None

    @classmethod
    def setup_class(cls):
        """setup any state specific to the execution of the given class (which
        usually contains tests).
        """
        openai_api_key = os.getenv("OPENAI_API_KEY", "")
        assert openai_api_key != "", "No \"OPENAI_API_KEY\" value set. Please set this env variable"
        cls.client = GPTClient(
            api_configuration_preset={
                "engine": "gpt-3.5-turbo",
                "api_key": openai_api_key,
                "max_attempts": 3,
                "wait_interval": 5
            },
            temperature=0,
            max_tokens=128,  # Will be much shorter in practice since the output format is constrained but not too small (
            # when explanations are used)
        )

    def test_classif(self):
        classifier = GPTZeroShotTextClassifier(
            gpt_client=self.client,
            input_desc="",
            classes=[
                {"class_name": "positive", "class_id": 1},
                {"class_name": "negative", "class_id": 2},
            ],
            output_desc="review polarity",
            examples=[],
            explanation_enabled=False,
        )
        response = classifier.get_response("This product is awsome! I live it!")
        assert response.predicted_class == "positive"

    def test_question_answering(self):
        answerer = GPTMultiQuestionAnswerer(
            gpt_client=self.client,
            questions=[
                ("country", "In which country do they live?"),
            ]
        )
        response = answerer.get_response("I'm a proud Parisian living in Paris.")
        assert "country" in response.parsed_answers
        assert "france" in response.parsed_answers["country"].lower(), f"unexpected answer {response.parsed_answers['country']}"

    def test_summarization(self):
        summarizer = GPTTextSummarizer(
            gpt_client=self.client,
            summary_length_limit_enabled=False
        )
        response = summarizer.get_response("For years I have used a different type of capo, this one is so much easier to use and does an excellent job!")
        assert len(response.summarized_text) > 0

    def test_generation(self):
        generator = GPTTextGenerator(
            gpt_client=self.client,
            task="translate in french",
            input_desc="text to be translated",
            examples=[],
            output_desc="",
            uses_input_text=True
        )
        response = generator.get_response("Hello my name is Alex.")
        assert len(response.generated_text) > 0
